@@ -1,44 +1,18 @@
-"""
-Formatting of UML elements like attributes, operations, stereotypes, etc.
-"""
+"""Formatting of UML elements like attributes, operations, stereotypes, etc."""
 
 import re
-from functools import singledispatch
-
-from gaphor.UML import uml2 as UML
-
-
-@singledispatch
-def format(el):
-    """
-    Format an UML element.
-    """
-    raise NotImplementedError(
-        "Format routine for type %s not implemented yet" % type(el)
-    )
-
-
-@format.register(UML.Property)
-def format_property(el, *args, **kwargs):
-    """
-    Format property or an association end.
-    """
-    if el.association and not (args or kwargs):
-        return format_association_end(el)
-    else:
-        return format_attribute(el, *args, **kwargs)
-
-
-def compile(regex):
-    return re.compile(regex, re.MULTILINE | re.S)
+from typing import Tuple
 
+from gaphor.core.format import format
+from gaphor.UML import uml as UML
 
 # Do not render if the name still contains a visibility element
-no_render_pat = compile(r"^\s*[+#-]")
+no_render_pat = re.compile(r"^\s*[+#-]", re.MULTILINE | re.S)
 vis_map = {"public": "+", "protected": "#", "package": "~", "private": "-"}
 
 
-def format_attribute(
+@format.register(UML.Property)
+def format_property(
     el,
     visibility=False,
     is_derived=False,
@@ -47,10 +21,8 @@ def format_attribute(
     default=False,
     tags=False,
 ):
-    """
-    Create a OCL representation of the attribute,
-    Returns the attribute as a string.
-    If one or more of the parameters (visibility, is_derived, type,
+    """Create a OCL representation of the attribute, Returns the attribute as a
+    string. If one or more of the parameters (visibility, is_derived, type,
     multiplicity, default and/or tags) is set, only that field is rendered.
     Note that the name of the attribute is always rendered, so a parseable
     string is returned.
@@ -59,10 +31,8 @@ def format_attribute(
     will not give you the same result.
     """
     name = el.name
-    if not name:
-        return name
 
-    if no_render_pat.match(name):
+    if name and no_render_pat.match(name):
         return name
 
     # Render all fields if they all are set to False
@@ -71,17 +41,20 @@ def format_attribute(
 
     s = []
 
-    if visibility:
+    if name and visibility:
         s.append(vis_map[el.visibility])
         s.append(" ")
 
-    if is_derived and el.isDerived:
+    if name and is_derived and el.isDerived:
         s.append("/")
 
-    s.append(name)
+    if name:
+        s.append(name)
 
     if type and el.typeValue:
         s.append(f": {el.typeValue}")
+    elif type and el.type and el.type.name:
+        s.append(f": {el.type.name}")
 
     if multiplicity:
         s.append(format_multiplicity(el))
@@ -96,10 +69,8 @@ def format_attribute(
     return "".join(s)
 
 
-def format_association_end(el):
-    """
-    Format association end.
-    """
+def format_association_end(el) -> Tuple[str, str]:
+    """Format association end."""
     name = ""
     n = []
     if el.name:
@@ -107,14 +78,11 @@ def format_association_end(el):
         n.append(" ")
         if el.isDerived:
             n.append("/")
-        if el.name:
-            n.append(el.name)
+        n.append(el.name)
 
         name = "".join(n)
 
-    m = []
-    m.append(format_multiplicity(el, bare=True))
-
+    m = [format_multiplicity(el, bare=True)]
     slots = [format(slot) for slot in el.appliedStereotype[:].slot if slot]
     if slots:
         m.append(" { %s }" % ",\n".join(slots))
@@ -134,10 +102,8 @@ def format_operation(
     tags=False,
     direction=False,
 ):
-    """
-    Create a OCL representation of the operation,
-    Returns the operation as a string.
-    """
+    """Create a OCL representation of the operation, Returns the operation as a
+    string."""
     name = el.name
     if not name:
         return ""
@@ -194,10 +160,6 @@ def format_parameter(
 
     if default and el.defaultValue:
         s.append(f" = {el.defaultValue}")
-    # if p.taggedValue:
-    #     tvs = ', '.join(filter(None, map(getattr, p.taggedValue,
-    #                                      ['value'] * len(p.taggedValue))))
-    #     s.append(' { %s }' % tvs)
     return "".join(s)
 
 
@@ -207,15 +169,39 @@ def format_slot(el):
 
 
 @format.register(UML.NamedElement)
-def format_namedelement(el):
-    """
-    Format named element.
-    """
+def format_namedelement(el, **kwargs):
+    """Format named element."""
     return el.name or ""
 
 
+@format.register(UML.MultiplicityElement)
 def format_multiplicity(el, bare=False):
     m = ""
     if el.upperValue:
         m = f"{el.lowerValue}..{el.upperValue}" if el.lowerValue else f"{el.upperValue}"
     return f"[{m}]" if m and not bare else m
+
+
+@format.register(UML.Relationship)
+def format_relationship(el):
+    return el.__class__.__name__
+
+
+@format.register(UML.Generalization)
+def format_generalization(el):
+    return f"general: {el.general and el.general.name or ''}"
+
+
+@format.register(UML.Dependency)
+def format_dependency(el):
+    return f"supplier: {el.supplier and el.supplier.name or ''}"
+
+
+@format.register(UML.Extend)
+def format_extend(el):
+    return f"extend: {el.extendedCase and el.extendedCase.name or ''}"
+
+
+@format.register(UML.Include)
+def format_include(el):
+    return f"include: {el.addition and el.addition.name or ''}"
