@@ -1,8 +1,7 @@
-"""
-Lexical analizer for attributes and operations.
+"""The lexical analyzer for attributes and operations.
 
-In this module some parse functions are added for attributes and operations.
-The regular expressions are constructed based on a series of
+In this module some parse functions are added for attributes and
+operations. The regular expressions are constructed based on a series of
 "sub-patterns". This makes it easy to identify the autonomy of an
 attribute/operation.
 """
@@ -10,20 +9,9 @@ attribute/operation.
 __all__ = ["parse_property", "parse_operation"]
 
 import re
-from functools import singledispatch
-
-import gaphor.UML.uml2 as uml2
-
-
-@singledispatch
-def parse(el, text):
-    """
-    Parser for an UML element.
-    """
-    raise NotImplementedError(
-        "Parsing routine for type %s not implemented yet" % type(el)
-    )
 
+from gaphor.core.format import parse
+from gaphor.UML import uml
 
 # Visibility (optional) ::= '+' | '-' | '#'
 vis_subpat = r"\s*(?P<vis>[-+#])?"
@@ -32,14 +20,14 @@ vis_subpat = r"\s*(?P<vis>[-+#])?"
 derived_subpat = r"\s*(?P<derived>/)?"
 
 # name (required) ::= name
-name_subpat = r"\s*(?P<name>[a-zA-Z_]\w*)"
+name_subpat = r"\s*(?P<name>[a-zA-Z_]\w*( +\w+)*)"
 
-# Multiplicity (added to type_subpat) ::= '[' [mult_l ..] mult_u ']'
+# Multiplicity ::= '[' [mult_l ..] mult_u ']'
 mult_subpat = r"\s*(\[\s*((?P<mult_l>[0-9]+)\s*\.\.)?\s*(?P<mult_u>([0-9]+|\*))\s*\])?"
 multa_subpat = r"\s*(\[?((?P<mult_l>[0-9]+)\s*\.\.)?\s*(?P<mult_u>([0-9]+|\*))\]?)?"
 
-# Type and multiplicity (optional) ::= ':' type [mult]
-type_subpat = r"\s*(:\s*(?P<type>\w+)\s*" + mult_subpat + r")?"
+# Type and multiplicity (optional) ::= ':' type
+type_subpat = r"\s*(:\s*(?P<type>[a-zA-Z_]\w*( +\w+)*))?"
 
 # default value (optional) ::= '=' default
 default_subpat = r"\s*(=\s*(?P<default>\S+))?"
@@ -72,6 +60,7 @@ attribute_pat = compile(
     + derived_subpat
     + name_subpat
     + type_subpat
+    + mult_subpat
     + default_subpat
     + tags_subpat
     + garbage_subpat
@@ -85,6 +74,7 @@ association_end_name_pat = compile(
     + vis_subpat
     + derived_subpat
     + name_subpat
+    + type_subpat
     + mult_subpat
     + ")?"
     + tags_subpat
@@ -103,6 +93,7 @@ operation_pat = compile(
     + name_subpat
     + params_subpat
     + type_subpat
+    + mult_subpat
     + tags_subpat
     + garbage_subpat
 )
@@ -114,17 +105,18 @@ parameter_pat = compile(
     + dir_subpat
     + name_subpat
     + type_subpat
+    + mult_subpat
     + default_subpat
     + tags_subpat
     + rest_subpat
 )
 
 # Lifeline:
-#  [name] [: type]
-lifeline_pat = compile("^" + name_subpat + type_subpat + garbage_subpat)
+#  [name] [: type]  # noqa: E800
+lifeline_pat = compile("^" + name_subpat + type_subpat + mult_subpat + garbage_subpat)
 
 
-def _set_visibility(el: uml2.Feature, vis: str):
+def _set_visibility(el: uml.Feature, vis: str):
     if vis == "+":
         el.visibility = "public"
     elif vis == "#":
@@ -140,10 +132,11 @@ def _set_visibility(el: uml2.Feature, vis: str):
             pass
 
 
-def parse_attribute(el: uml2.Property, s: str) -> None:
-    """
-    Parse string s in the property. Tagged values, multiplicity and stuff
-    like that is altered to reflect the data in the property string.
+def parse_attribute(el: uml.Property, s: str) -> None:
+    """Parse string s in the property.
+
+    Tagged values, multiplicity and stuff like that is altered to
+    reflect the data in the property string.
     """
     m = attribute_pat.match(s)
     if not m or m.group("garbage"):
@@ -160,7 +153,6 @@ def parse_attribute(el: uml2.Property, s: str) -> None:
             el.defaultValue = None
     else:
         g = m.group
-        el.model.create
         _set_visibility(el, g("vis"))
         el.isDerived = g("derived") and True or False
         el.name = g("name")
@@ -170,11 +162,11 @@ def parse_attribute(el: uml2.Property, s: str) -> None:
         el.defaultValue = g("default")
 
 
-def parse_association_end(el: uml2.Property, s: str) -> None:
-    """
-    Parse the text at one end of an association. The association end holds
-    two strings. It is automatically figured out which string is fed to the
-    parser.
+def parse_association_end(el: uml.Property, s: str) -> None:
+    """Parse the text at one end of an association.
+
+    The association end holds two strings. It is automatically figured
+    out which string is fed to the parser.
     """
     el.model.create
 
@@ -186,9 +178,8 @@ def parse_association_end(el: uml2.Property, s: str) -> None:
 
     # clear also multiplicity if no characters in ``s``
     m = association_end_mult_pat.match(s)
-    if m and not m.group("mult_u"):
-        if el.upperValue:
-            el.upperValue = None
+    if m and not m.group("mult_u") and el.upperValue:
+        el.upperValue = None
 
     if m and m.group("mult_u") or m.group("tags"):
         g = m.group
@@ -215,33 +206,34 @@ def parse_association_end(el: uml2.Property, s: str) -> None:
                 el.upperValue = g("mult_u")
 
 
-@parse.register(uml2.Property)
-def parse_property(el: uml2.Property, s: str) -> None:
+@parse.register(uml.Property)
+def parse_property(el: uml.Property, s: str) -> None:
     if el.association:
         parse_association_end(el, s)
     else:
         parse_attribute(el, s)
 
 
-@parse.register(uml2.Operation)
-def parse_operation(el: uml2.Operation, s: str) -> None:
-    """
-    Parse string s in the operation. Tagged values, parameters and
-    visibility is altered to reflect the data in the operation string.
+@parse.register(uml.Operation)
+def parse_operation(el: uml.Operation, s: str) -> None:
+    """Parse string s in the operation.
+
+    Tagged values, parameters and visibility is altered to reflect the
+    data in the operation string.
     """
     m = operation_pat.match(s)
     if not m or m.group("garbage"):
         el.name = s
         del el.visibility
-        list(map(uml2.Parameter.unlink, list(el.returnResult)))
-        list(map(uml2.Parameter.unlink, list(el.formalParameter)))
+        list(map(uml.Parameter.unlink, list(el.returnResult)))
+        list(map(uml.Parameter.unlink, list(el.formalParameter)))
     else:
         g = m.group
         create = el.model.create
         _set_visibility(el, g("vis"))
         el.name = g("name")
         if not el.returnResult:
-            el.returnResult = create(uml2.Parameter)
+            el.returnResult = create(uml.Parameter)
         p = el.returnResult[0]
         p.direction = "return"
         p.typeValue = g("type")
@@ -258,7 +250,7 @@ def parse_operation(el: uml2.Operation, s: str) -> None:
             try:
                 p = el.formalParameter[pindex]
             except IndexError:
-                p = create(uml2.Parameter)
+                p = create(uml.Parameter)
             p.direction = g("dir") or "in"
             p.name = g("name")
             p.typeValue = g("type")
@@ -276,11 +268,11 @@ def parse_operation(el: uml2.Operation, s: str) -> None:
             fp.unlink()
 
 
-def parse_lifeline(el: uml2.Lifeline, s: str) -> None:
-    """
-    Parse string s in a lifeline. If a class is defined and can be found
-    in the datamodel, then a class is connected to the lifelines 'represents'
-    property.
+def parse_lifeline(el: uml.Lifeline, s: str) -> None:
+    """Parse string s in a lifeline.
+
+    If a class is defined and can be found in the datamodel, then a
+    class is connected to the lifelines 'represents' property.
     """
     m = lifeline_pat.match(s)
     g = m.group
@@ -292,18 +284,15 @@ def parse_lifeline(el: uml2.Lifeline, s: str) -> None:
         if t:
             el.name += ": " + t
         # In the near future the data model should be extended with
-        # Lifeline.represents: ConnectableElement
+        # Lifeline.represents: ConnectableElement  # noqa: E800
 
 
-def render_lifeline(el: uml2.Lifeline) -> str:
-    """
-    """
+def render_lifeline(el: uml.Lifeline) -> str:
+    """"""
     return el.name or ""
 
 
-@parse.register(uml2.NamedElement)
-def parse_namedelement(el: uml2.NamedElement, text: str) -> None:
-    """
-    Parse named element by simply assigning text to its name.
-    """
+@parse.register(uml.NamedElement)
+def parse_namedelement(el: uml.NamedElement, text: str) -> None:
+    """Parse named element by simply assigning text to its name."""
     el.name = text
