@@ -1,27 +1,16 @@
-"""Support for actions in generic files.
+"""Support for actions in generic files."""
 
-"""
-
-import platform
-from typing import Optional, Sequence, get_type_hints
-
-_primary = "⌘" if platform.system() == "Darwin" else "Ctrl"
-
-
-def primary():
-    global _primary
-    return _primary
+from typing import get_type_hints
 
 
 class action:
-    """
-    Decorator. Turns a regular function (/method) into a full blown
-    Action class.
+    """Decorator. Turns a regular function (/method) into a full blown Action
+    class.
 
     >>> class A:
     ...     @action(name="my_action", label="my action")
     ...     def myaction(self):
-    ...         print('action called')
+    ...         print("action called")
     >>> a = A()
     >>> a.myaction()
     action called
