@@ -1,18 +1,9 @@
-import pytest
-
-from gaphor import UML
+from gaphor.core.modeling import Comment
 from gaphor.diagram.general.generalpropertypages import CommentItemPropertyPage
-from gaphor.services.eventmanager import EventManager
-from gaphor.UML.elementfactory import ElementFactory
-
-
-@pytest.fixture
-def element_factory():
-    return ElementFactory(EventManager())
 
 
 def test_property_page_construction(element_factory):
-    comment = element_factory.create(UML.Comment)
+    comment = element_factory.create(Comment)
     prop_page = CommentItemPropertyPage(comment)
 
     widget = prop_page.construct()
