@@ -1,13 +1,11 @@
-"""
-This plugin extends Gaphor with XMI export functionality.
-"""
+"""This plugin extends Gaphor with XMI export functionality."""
 
 import logging
 
 from gaphor.abc import ActionProvider, Service
 from gaphor.core import action, gettext
 from gaphor.plugins.xmiexport import exportmodel
-from gaphor.ui.filedialog import FileDialog
+from gaphor.ui.filedialog import save_file_dialog
 
 logger = logging.getLogger(__name__)
 
@@ -28,17 +26,14 @@ class XMIExport(Service, ActionProvider):
     )
     def execute(self):
         filename = self.file_manager.filename
-        if filename:
-            filename = filename.replace(".gaphor", ".xmi")
-        else:
-            filename = "model.xmi"
-
-        file_dialog = FileDialog(
-            gettext("Export model to XMI file"), action="save", filename=filename
+        filename = filename.replace(".gaphor", ".xmi") if filename else "model.xmi"
+        filename = save_file_dialog(
+            gettext("Export model to XMI file"),
+            filename=filename,
+            extension=".xmi",
+            filters=[(gettext("All XMI Files"), ".xmi", "text/xml")],
         )
 
-        filename = file_dialog.selection
-
         if filename and len(filename) > 0:
             logger.debug(f"Exporting XMI model to: {filename}")
             export = exportmodel.XMIExport(self.element_factory)
