@@ -1,19 +1,19 @@
-"""
-A registry for components (e.g. services) and event handling.
-"""
+"""A registry for components (e.g. services) and event handling."""
 
 from typing import Iterator, Set, Tuple, Type, TypeVar
 
 from gaphor.abc import Service
-from gaphor.application import ComponentLookupError
 
 T = TypeVar("T", bound=Service)
 
 
+class ComponentLookupError(LookupError):
+    pass
+
+
 class ComponentRegistry(Service):
-    """
-    The ComponentRegistry provides a home for application wide components.
-    """
+    """The ComponentRegistry provides a home for application wide
+    components."""
 
     def __init__(self) -> None:
         self._comp: Set[Tuple[str, object]] = set()
@@ -23,6 +23,7 @@ class ComponentRegistry(Service):
 
     def get_service(self, name: str) -> Service:
         """Obtain a service used by Gaphor by name.
+
         E.g. service("element_factory")
         """
         return self.get(Service, name)  # type: ignore[misc] # noqa: F821
@@ -39,7 +40,7 @@ class ComponentRegistry(Service):
             raise ComponentLookupError(
                 f"More than one component matches {base}+{name}: {found}"
             )
-        if len(found) == 0:
+        if not found:
             raise ComponentLookupError(
                 f"Component with type {base} and name {name} is not registered"
             )
