@@ -1,10 +1,13 @@
-"""About and help services. (help browser anyone?)"""
+"""About and help services.
+
+(help browser anyone?)
+"""
 
 
 import importlib
 
 import importlib_metadata
-from gi.repository import GdkPixbuf, Gtk
+from gi.repository import Gtk
 
 from gaphor.abc import ActionProvider, Service
 from gaphor.core import action
