@@ -1,27 +1,24 @@
-"""
-Load and save Gaphor models to Gaphors own XML format.
+"""Load and save Gaphor models to Gaphors own XML format.
 
-Three functions are exported:
-load(filename)
-    load a model from a file
-save(filename)
-    store the current model in a file
+Three functions are exported: load(filename)     load a model from a
+file save(filename)     store the current model in a file
 """
 
 __all__ = ["load", "save"]
 
-import gc
 import io
 import logging
 import os.path
 import uuid
+from functools import partial
 
-import gaphas
-
-from gaphor import UML, application
-from gaphor.i18n import gettext
-from gaphor.storage import diagramitems, parser
-from gaphor.UML.collection import collection
+from gaphor import application
+from gaphor.core.modeling.collection import collection
+from gaphor.core.modeling.diagram import Diagram, PseudoCanvas
+from gaphor.core.modeling.element import Element
+from gaphor.core.modeling.presentation import Presentation
+from gaphor.core.modeling.stylesheet import StyleSheet
+from gaphor.storage import parser
 
 FILE_FORMAT_VERSION = "3.0"
 NAMESPACE_MODEL = "http://gaphor.sourceforge.net/model"
@@ -35,98 +32,9 @@ def save(writer=None, factory=None, status_queue=None):
             status_queue(status)
 
 
-def save_generator(writer, factory):  # noqa: C901
-    """
-    Save the current model using @writer, which is a
-    gaphor.misc.xmlwriter.XMLWriter instance.
-    """
-
-    def save_reference(name, value):
-        """
-        Save a value as a reference to another element in the model.
-        This applies to both UML as well as canvas items.
-        """
-        if value.id:
-            writer.startElement(name, {})
-            writer.startElement("ref", {"refid": value.id})
-            writer.endElement("ref")
-            writer.endElement(name)
-
-    def save_collection(name, value):
-        """
-        Save a list of references.
-        """
-        if len(value) > 0:
-            writer.startElement(name, {})
-            writer.startElement("reflist", {})
-            for v in value:
-                if v.id:
-                    writer.startElement("ref", {"refid": v.id})
-                    writer.endElement("ref")
-            writer.endElement("reflist")
-            writer.endElement(name)
-
-    def save_value(name, value):
-        """
-        Save a value (attribute).
-        """
-        if value is not None:
-            writer.startElement(name, {})
-            writer.startElement("val", {})
-            if isinstance(value, str):
-                writer.characters(value)
-            elif isinstance(value, bool):
-                # Write booleans as 0/1.
-                writer.characters(str(int(value)))
-            else:
-                writer.characters(str(value))
-            writer.endElement("val")
-            writer.endElement(name)
-
-    def save_element(name, value):
-        """
-        Save attributes and references from items in the gaphor.UML module.
-        A value may be a primitive (string, int), a gaphor.UML.collection
-        (which contains a list of references to other UML elements) or a
-        gaphas.Canvas (which contains canvas items).
-        """
-        if isinstance(value, (UML.Element, gaphas.Item)):
-            save_reference(name, value)
-        elif isinstance(value, collection):
-            save_collection(name, value)
-        elif isinstance(value, gaphas.Canvas):
-            writer.startElement("canvas", {})
-            value.save(save_canvasitem)
-            writer.endElement("canvas")
-        else:
-            save_value(name, value)
-
-    def save_canvasitem(name, value, reference=False):
-        """
-        Save attributes and references in a gaphor.diagram.* object.
-        The extra attribute reference can be used to force UML
-        """
-        if isinstance(value, collection) or (
-            isinstance(value, (list, tuple)) and reference is True
-        ):
-            save_collection(name, value)
-        elif reference:
-            save_reference(name, value)
-        elif isinstance(value, gaphas.Item):
-            writer.startElement(
-                "item", {"id": value.id, "type": value.__class__.__name__}
-            )
-            value.save(save_canvasitem)
-
-            for child in value.canvas.get_children(value):
-                save_canvasitem(None, child)
-
-            writer.endElement("item")
-
-        elif isinstance(value, UML.Element):
-            save_reference(name, value)
-        else:
-            save_value(name, value)
+def save_generator(writer, factory):
+    """Save the current model using @writer, which is a
+    gaphor.storage.xmlwriter.XMLWriter instance."""
 
     writer.startDocument()
     writer.startPrefixMapping("", NAMESPACE_MODEL)
@@ -140,15 +48,13 @@ def save_generator(writer, factory):  # noqa: C901
     )
 
     size = factory.size()
-    n = 0
-    for e in list(factory.values()):
+    for n, e in enumerate(factory.values(), start=1):
         clazz = e.__class__.__name__
         assert e.id
         writer.startElement(clazz, {"id": str(e.id)})
-        e.save(save_element)
+        e.save(partial(save_element, writer=writer))
         writer.endElement(clazz)
 
-        n += 1
         if n % 25 == 0:
             yield (n * 100) / size
 
@@ -157,14 +63,98 @@ def save_generator(writer, factory):  # noqa: C901
     writer.endDocument()
 
 
-def load_elements(elements, factory, gaphor_version="1.0.0"):
-    for status in load_elements_generator(elements, factory, gaphor_version):
+def save_element(name, value, writer):
+    """Save attributes and references from items in the gaphor.UML module.
+
+    A value may be a primitive (string, int), a
+    gaphor.core.modeling.collection (which contains a list of references
+    to other UML elements) or a Diagram (which contains diagram items).
+    """
+
+    def save_reference(name, value):
+        """Save a value as a reference to another element in the model.
+
+        This applies to both UML as well as canvas items.
+        """
+        if value.id:
+            writer.startElement(name, {})
+            writer.startElement("ref", {"refid": value.id})
+            writer.endElement("ref")
+            writer.endElement(name)
+
+    def save_collection(name, value):
+        """Save a list of references."""
+        if value:
+            writer.startElement(name, {})
+            writer.startElement("reflist", {})
+            for v in value:
+                if v.id:
+                    writer.startElement("ref", {"refid": v.id})
+                    writer.endElement("ref")
+            writer.endElement("reflist")
+            writer.endElement(name)
+
+    def save_value(name, value):
+        """Save a value (attribute)."""
+        if value is not None:
+            writer.startElement(name, {})
+            writer.startElement("val", {})
+            if isinstance(value, bool):
+                # Write booleans as 0/1.
+                writer.characters(str(int(value)))
+            else:
+                writer.characters(str(value))
+            writer.endElement("val")
+            writer.endElement(name)
+
+    def save_diagram(value):
+        """Save attributes and references in a gaphor.diagram.* object.
+
+        The extra attribute reference can be used to force UML
+        """
+        assert isinstance(value, Presentation)
+        writer.startElement("item", {"id": value.id, "type": value.__class__.__name__})
+        value.save(save_diagram_item)
+
+        for child in value.children:
+            save_diagram(child)
+
+        writer.endElement("item")
+
+    def save_diagram_item(name, value):
+        """Save attributes and references in a gaphor.diagram.* object.
+
+        The extra attribute reference can be used to force UML
+        """
+        if isinstance(value, collection):
+            save_collection(name, value)
+        elif isinstance(value, Element):
+            save_reference(name, value)
+        else:
+            save_value(name, value)
+
+    if isinstance(value, Element):
+        save_reference(name, value)
+    elif isinstance(value, collection):
+        save_collection(name, value)
+    elif isinstance(value, PseudoCanvas):
+        writer.startElement("canvas", {})
+        value.save(save_diagram)
+        writer.endElement("canvas")
+    else:
+        save_value(name, value)
+
+
+def load_elements(elements, factory, modeling_language, gaphor_version="1.0.0"):
+    for _ in load_elements_generator(
+        elements, factory, modeling_language, gaphor_version
+    ):
         pass
 
 
-def load_elements_generator(elements, factory, gaphor_version):
-    """
-    Load a file and create a model if possible.
+def load_elements_generator(elements, factory, modeling_language, gaphor_version):
+    """Load a file and create a model if possible.
+
     Exceptions: IOError, ValueError.
     """
     log.debug(f"Loading {len(elements)} elements")
@@ -180,28 +170,23 @@ def load_elements_generator(elements, factory, gaphor_version):
     # First create elements and canvas items in the factory
     # The elements are stored as attribute 'element' on the parser objects:
     yield from _load_elements_and_canvasitems(
-        elements, factory, gaphor_version, update_status_queue
+        elements, factory, modeling_language, gaphor_version, update_status_queue
     )
     yield from _load_attributes_and_references(elements, update_status_queue)
 
-    for d in factory.select(lambda e: isinstance(e, UML.Diagram)):
-        canvas = d.canvas
-        # update_now() is implicitly called when lock is released
-        canvas.block_updates = False
+    ensure_style_sheet_is_present(factory)
 
-    # do a postload:
     for id, elem in list(elements.items()):
         yield from update_status_queue()
         elem.element.postload()
 
 
 def _load_elements_and_canvasitems(
-    elements, factory, gaphor_version, update_status_queue
+    elements, factory, modeling_language, gaphor_version, update_status_queue
 ):
     def create_canvasitems(diagram, canvasitems, parent=None):
-        """
-        Canvas is a read gaphas.Canvas, items is a list of parser.canvasitem's
-        """
+        """Diagram is a Core Diagram, items is a list of
+        parser.canvasitem's."""
         if version_lower_than(gaphor_version, (1, 1, 0)):
             new_canvasitems = upgrade_message_item_to_1_1_0(canvasitems)
             canvasitems.extend(new_canvasitems)
@@ -210,20 +195,25 @@ def _load_elements_and_canvasitems(
 
         for item in canvasitems:
             item = upgrade_canvas_item_to_1_0_2(item)
+            item = upgrade_canvas_item_to_1_3_0(item)
             if version_lower_than(gaphor_version, (1, 1, 0)):
                 item = upgrade_presentation_item_to_1_1_0(item)
-            cls = getattr(diagramitems, item.type)
+            cls = modeling_language.lookup_diagram_item(item.type)
+            assert cls, f"No diagram item for type {item.type}"
             item.element = diagram.create_as(cls, item.id, parent=parent)
             create_canvasitems(diagram, item.canvasitems, parent=item.element)
 
     for id, elem in list(elements.items()):
         yield from update_status_queue()
         if isinstance(elem, parser.element):
-            cls = getattr(UML, elem.type)
+            if version_lower_than(gaphor_version, (2, 1, 0)):
+                elem = upgrade_element_to_2_1_0(elem)
+
+            cls = modeling_language.lookup_element(elem.type)
+            assert cls, f"Type {elem.type} can not be loaded: no such element"
             elem.element = factory.create_as(cls, id)
-            if isinstance(elem.element, UML.Diagram):
+            if isinstance(elem.element, Diagram):
                 assert elem.canvas
-                elem.element.canvas.block_updates = True
                 create_canvasitems(elem.element, elem.canvas.canvasitems)
         elif not isinstance(elem, parser.canvasitem):
             raise ValueError(
@@ -261,20 +251,20 @@ def _load_attributes_and_references(elements, update_status_queue):
                     elem.element.load(name, ref.element)
 
 
-def load(filename, factory, status_queue=None):
-    """
-    Load a file and create a model if possible.
+def load(filename, factory, modeling_language, status_queue=None):
+    """Load a file and create a model if possible.
+
     Optionally, a status queue function can be given, to which the
     progress is written (as status_queue(progress)).
     """
-    for status in load_generator(filename, factory):
+    for status in load_generator(filename, factory, modeling_language):
         if status_queue:
             status_queue(status)
 
 
-def load_generator(filename, factory):
-    """
-    Load a file and create a model if possible.
+def load_generator(filename, factory, modeling_language):
+    """Load a file and create a model if possible.
+
     This function is a generator. It will yield values from 0 to 100 (%)
     to indicate its progression.
     """
@@ -305,17 +295,15 @@ def load_generator(filename, factory):
     log.info(f"Read {len(elements)} elements from file")
 
     factory.flush()
-    gc.collect()
     with factory.block_events():
         try:
             for percentage in load_elements_generator(
-                elements, factory, gaphor_version
+                elements, factory, modeling_language, gaphor_version
             ):
                 if percentage:
                     yield percentage / 2 + 50
                 else:
                     yield percentage
-            gc.collect()
             yield 100
         except Exception as e:
             log.warning(f"file {filename} could not be loaded ({e})")
@@ -324,12 +312,10 @@ def load_generator(filename, factory):
 
 
 def version_lower_than(gaphor_version, version):
-    """
-    Only major and minor versions are checked.
+    """Only major and minor versions are checked.
 
-    >>> version_lower_than('0.3.0', (0, 15, 0))
+    >>> version_lower_than("0.3.0", (0, 15, 0))
     True
-
     """
     parts = gaphor_version.split(".")
 
@@ -382,13 +368,16 @@ def clone_canvasitem(item, subject_id):
     return new_item
 
 
+def ensure_style_sheet_is_present(factory):
+    style_sheet = next(factory.select(StyleSheet), None)
+    if not style_sheet:
+        factory.create(StyleSheet)
+
+
 def upgrade_message_item_to_1_1_0(canvasitems):
-    """
-    Create new MessageItem's for each `message` and `inverted` message.
-    """
+    """Create new MessageItem's for each `message` and `inverted` message."""
     new_canvasitems = []
     for item in canvasitems:
-        # new_canvasitems.append(item)
         if item.type == "MessageItem" and item.references.get("subject"):
             messages = item.references.get("message", [])
             inverted = item.references.get("inverted", [])
@@ -411,3 +400,17 @@ def upgrade_message_item_to_1_1_0(canvasitems):
                     new_item.references["head-connection"],
                 )
     return new_canvasitems
+
+
+def upgrade_canvas_item_to_1_3_0(item):
+    if item.type in ("InitialPseudostateItem", "HistoryPseudostateItem"):
+        item.type = "PseudostateItem"
+    return item
+
+
+def upgrade_element_to_2_1_0(elem):
+    for name, refids in dict(elem.references).items():
+        if name == "ownedComment":
+            elem.references["comment"] = refids
+            del elem.references["ownedComment"]
+    return elem
