@@ -1,6 +1,4 @@
-"""
-UI related events.
-"""
+"""UI related events."""
 
 
 class DiagramOpened:
@@ -20,6 +18,11 @@ class DiagramSelectionChanged:
         self.selected_items = selected_items
 
 
+class ModelingLanguageChanged:
+    def __init__(self, modeling_language):
+        self.modeling_language = modeling_language
+
+
 class FileLoaded:
     def __init__(self, service, filename=None):
         self.service = service
@@ -30,3 +33,8 @@ class FileSaved:
     def __init__(self, service, filename=None):
         self.service = service
         self.filename = filename
+
+
+class Notification:
+    def __init__(self, message):
+        self.message = message
