@@ -1,23 +1,18 @@
-"""
-The file service is responsible for loading and saving the user data.
-"""
+"""The file service is responsible for loading and saving the user data."""
 
 import logging
-from typing import Optional
 
 from gi.repository import Gtk
 
-import gaphor.ui
-from gaphor import UML
 from gaphor.abc import ActionProvider, Service
 from gaphor.core import action, event_handler, gettext
 from gaphor.event import SessionShutdown, SessionShutdownRequested
-from gaphor.misc.errorhandler import error_handler
-from gaphor.misc.gidlethread import GIdleThread, Queue, QueueEmpty, QueueFull
-from gaphor.misc.xmlwriter import XMLWriter
 from gaphor.storage import storage, verify
+from gaphor.storage.xmlwriter import XMLWriter
+from gaphor.ui.errorhandler import error_handler
 from gaphor.ui.event import FileLoaded, FileSaved
-from gaphor.ui.filedialog import FileDialog
+from gaphor.ui.filedialog import GAPHOR_FILTER, save_file_dialog
+from gaphor.ui.gidlethread import GIdleThread, Queue
 from gaphor.ui.questiondialog import QuestionDialog
 from gaphor.ui.statuswindow import StatusWindow
 
@@ -27,17 +22,36 @@ MAX_RECENT = 10
 log = logging.getLogger(__name__)
 
 
-class FileManager(Service, ActionProvider):
-    """
-    The file service, responsible for loading and saving Gaphor models.
-    """
+def error_message(e):
+    if not isinstance(e, IOError):
+        return gettext(
+            "Gaphor was not able to store the model, probably due to an internal error:\n{exc}\nIf you think this is a bug, please contact the developers."
+        ).format(exc=str(e))
+    if e.errno == 13:
+        return gettext(
+            "You do not have the permissions necessary to save the model.\nPlease check that you typed the location correctly and try again."
+        )
+    elif e.errno == 28:
+        return gettext(
+            "You do not have enough free space on the device to save the model.\nPlease free up some disk space and try again or save it in a different location."
+        )
+    return gettext(
+        "The model cannot be stored at this location:\n{exc}\nPlease check that you typed the location correctly and try again."
+    ).format(exc=str(e))
 
-    def __init__(self, event_manager, element_factory, main_window, properties):
-        """File manager constructor.  There is no current filename yet."""
+
+class FileManager(Service, ActionProvider):
+    """The file service, responsible for loading and saving Gaphor models."""
+
+    def __init__(self, event_manager, element_factory, modeling_language, main_window):
+        """File manager constructor.
+
+        There is no current filename yet.
+        """
         self.event_manager = event_manager
         self.element_factory = element_factory
+        self.modeling_language = modeling_language
         self.main_window = main_window
-        self.properties = properties
         self._filename = None
 
         event_manager.subscribe(self._on_session_shutdown_request)
@@ -47,37 +61,32 @@ class FileManager(Service, ActionProvider):
         self.event_manager.unsubscribe(self._on_session_shutdown_request)
 
     def get_filename(self):
-        """Return the current file name.  This method is used by the filename
-        property."""
+        """Return the current file name.
+
+        This method is used by the filename property.
+        """
         return self._filename
 
     def set_filename(self, filename):
-        """Sets the current file name.  This method is used by the filename
-        property. Setting the current filename will update the recent file
-        list."""
+        """Sets the current file name.
+
+        This method is used by the filename property. Setting the
+        current filename will update the recent file list.
+        """
 
         if filename != self._filename:
             self._filename = filename
 
     filename = property(get_filename, set_filename)
 
-    def new(self):
-        element_factory = self.element_factory
-        element_factory.flush()
-        with element_factory.block_events():
-            model = element_factory.create(UML.Package)
-            model.name = gettext("New model")
-            diagram = element_factory.create(UML.Diagram)
-            diagram.package = model
-            diagram.name = gettext("main")
-        self.filename = None
-        element_factory.model_ready()
-
     def load(self, filename):
-        """Load the Gaphor model from the supplied file name.  A status window
-        displays the loading progress.  The load generator updates the progress
-        queue.  The loader is passed to a GIdleThread which executes the load
-        generator.  If loading is successful, the filename is set."""
+        """Load the Gaphor model from the supplied file name.
+
+        A status window displays the loading progress.  The load
+        generator updates the progress queue.  The loader is passed to a
+        GIdleThread which executes the load generator.  If loading is
+        successful, the filename is set.
+        """
 
         queue = Queue()
         status_window = StatusWindow(
@@ -89,7 +98,7 @@ class FileManager(Service, ActionProvider):
 
         try:
             loader = storage.load_generator(
-                filename.encode("utf-8"), self.element_factory
+                filename.encode("utf-8"), self.element_factory, self.modeling_language
             )
             worker = GIdleThread(loader, queue)
 
@@ -101,25 +110,32 @@ class FileManager(Service, ActionProvider):
 
             self.filename = filename
             self.event_manager.handle(FileLoaded(self, filename))
-        except (QueueEmpty, QueueFull):
+        except Exception:
             error_handler(
-                message=gettext(
-                    "Error while loading model from file {filename}"
-                ).format(filename=filename)
+                message=gettext("Unable to open model “{filename}”.").format(
+                    filename=filename
+                ),
+                secondary_message=gettext(
+                    "This file does not contain a valid Gaphor model."
+                ),
+                window=self.main_window.window,
             )
             raise
         finally:
             status_window.destroy()
 
     def verify_orphans(self):
-        """Verify that no orphaned elements are saved.  This method checks
-        of there are any orphan references in the element factory.  If orphans
-        are found, a dialog is displayed asking the user if it is OK to
-        unlink them."""
+        """Verify that no orphaned elements are saved.
+
+        This method checks of there are any orphan references in the
+        element factory.  If orphans are found, a dialog is displayed
+        asking the user if it is OK to unlink them.
+        """
 
         orphans = verify.orphan_references(self.element_factory)
 
         if orphans:
+            log.info("Found orphan references %s", orphans)
             main_window = self.main_window
 
             dialog = QuestionDialog(
@@ -136,28 +152,19 @@ class FileManager(Service, ActionProvider):
                 for orphan in orphans:
                     orphan.unlink()
 
-    def verify_filename(self, filename):
-        """Verify that the supplied filename is using the proper default
-        extension.  If not, the extension is added to the filename
-        and returned."""
-
-        if not filename.endswith(DEFAULT_EXT):
-            filename = filename + DEFAULT_EXT
-
-        return filename
-
     def save(self, filename):
-        """Save the current UML model to the specified file name.  Before
-        writing the model file, this will verify that there are no orphan
-        references.  It will also verify that the filename has the correct
-        extension.  A status window is displayed while the GIdleThread
-        is executed.  This thread actually saves the model."""
+        """Save the current UML model to the specified file name.
 
-        if not filename or not len(filename):
+        Before writing the model file, this will verify that there are
+        no orphan references.  It will also verify that the filename has
+        the correct extension.  A status window is displayed while the
+        GIdleThread is executed.  This thread actually saves the model.
+        """
+
+        if not (filename and len(filename)):
             return
 
         self.verify_orphans()
-        filename = self.verify_filename(filename)
 
         main_window = self.main_window
         queue = Queue()
@@ -179,11 +186,13 @@ class FileManager(Service, ActionProvider):
 
             self.filename = filename
             self.event_manager.handle(FileSaved(self, filename))
-        except (OSError, QueueEmpty, QueueFull):
+        except Exception as e:
             error_handler(
-                message=gettext("Error while saving model to file {filename}").format(
+                message=gettext("Unable to save model “{filename}”.").format(
                     filename=filename
-                )
+                ),
+                secondary_message=error_message(e),
+                window=self.main_window.window,
             )
             raise
         finally:
@@ -191,8 +200,7 @@ class FileManager(Service, ActionProvider):
 
     @action(name="file-save", shortcut="<Primary>s")
     def action_save(self):
-        """
-        Save the file. Depending on if there is a file name, either perform
+        """Save the file. Depending on if there is a file name, either perform
         the save directly or present the user with a save dialog box.
 
         Returns True if the saving actually succeeded.
@@ -200,29 +208,28 @@ class FileManager(Service, ActionProvider):
 
         filename = self.filename
 
-        if filename:
-            self.save(filename)
-            return True
-        else:
+        if not filename:
             return self.action_save_as()
 
+        self.save(filename)
+        return True
+
     @action(name="file-save-as", shortcut="<Primary><Shift>s")
     def action_save_as(self):
-        """
-        Save the model in the element_factory by allowing the
-        user to select a file name.
+        """Save the model in the element_factory by allowing the user to select
+        a file name.
 
         Returns True if the saving actually happened.
         """
 
-        file_dialog = FileDialog(
-            gettext("Save Gaphor Model As"), action="save", filename=self.filename
+        filename = save_file_dialog(
+            gettext("Save Gaphor Model As"),
+            parent=self.main_window.window,
+            filename=self.filename,
+            extension=".gaphor",
+            filters=GAPHOR_FILTER,
         )
 
-        filename = file_dialog.selection
-
-        file_dialog.destroy()
-
         if filename:
             self.save(filename)
             return True
@@ -231,8 +238,8 @@ class FileManager(Service, ActionProvider):
 
     @event_handler(SessionShutdownRequested)
     def _on_session_shutdown_request(self, event):
-        """
-        Ask user to close window if the model has changed.
+        """Ask user to close window if the model has changed.
+
         The user is asked to either discard the changes, keep the
         application running or save the model and quit afterwards.
         """
@@ -245,12 +252,12 @@ class FileManager(Service, ActionProvider):
                 self.main_window.window,
                 Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                 Gtk.MessageType.WARNING,
-                Gtk.ButtonsType.NONE,
-                gettext("Save changed to your model before closing?"),
             )
-            dialog.format_secondary_text(
-                gettext("If you close without saving, your changes will be discarded.")
+            dialog.props.text = gettext("Save changed to your model before closing?")
+            dialog.props.secondary_text = gettext(
+                "If you close without saving, your changes will be discarded."
             )
+
             dialog.add_buttons(
                 gettext("Close _without saving"),
                 Gtk.ResponseType.REJECT,
