@@ -1,6 +1,4 @@
-"""
-Layout code from a simple XML description.
-"""
+"""Layout code from a simple XML description."""
 
 from typing import Callable, Dict
 from xml.etree.ElementTree import fromstring
@@ -10,11 +8,12 @@ from gi.repository import Gtk
 widget_factory: Dict[str, Callable] = {}
 
 
-def deserialize(layout, container, layoutstr, itemfactory):
-    """
-    Return a new layout with it's attached frames. Frames that should be floating
-    already have their Gtk.Window attached (check frame.get_parent()). Transient settings
-    and such should be done by the invoking application.
+def deserialize(container, layoutstr, itemfactory, properties):
+    """Return a new layout with it's attached frames.
+
+    Frames that should be floating already have their Gtk.Window
+    attached (check frame.get_parent()). Transient settings and such
+    should be done by the invoking application.
     """
 
     def _des(element, index, parent_widget=None):
@@ -26,15 +25,22 @@ def deserialize(layout, container, layoutstr, itemfactory):
             add(widget, index, parent_widget, resize)
         else:
             factory = widget_factory[element.tag]
-            widget = factory(parent=parent_widget, index=index, **element.attrib)
+            widget = factory(
+                parent=parent_widget,
+                index=index,
+                properties=properties,
+                **element.attrib,
+            )
             assert widget, f"No widget ({widget})"
             for i, e in enumerate(element):
                 _des(e, i, widget)
         return widget
 
+    layout = []
     tree = fromstring(layoutstr)
     for index, element in enumerate(tree):
         layout.append(_des(element, index, container))
+    return layout
 
 
 def add(widget, index, parent_widget, resize=False, shrink=False):
@@ -44,15 +50,13 @@ def add(widget, index, parent_widget, resize=False, shrink=False):
         elif index == 1:
             parent_widget.pack2(child=widget, resize=resize, shrink=shrink)
     elif isinstance(parent_widget, Gtk.Box):
-        parent_widget.pack_start(widget, resize, resize, 2)
+        parent_widget.pack_start(widget, resize, resize, 0)
     else:
         parent_widget.add(widget)
 
 
 def factory(typename):
-    """
-    Simple decorator for populating the widget_factory dictionary.
-    """
+    """Simple decorator for populating the widget_factory dictionary."""
 
     def _factory(func):
         widget_factory[typename] = func
@@ -61,22 +65,27 @@ def factory(typename):
     return _factory
 
 
+def _position_changed(paned, _gparam, properties):
+    properties.set(paned.get_name(), paned.props.position)
+
+
 @factory("paned")
-def paned(parent, index, orientation, position=None):
+def paned(parent, index, properties, name, orientation, position=None):
     paned = Gtk.Paned.new(
         Gtk.Orientation.HORIZONTAL
         if orientation == "horizontal"
         else Gtk.Orientation.VERTICAL
     )
+    paned.set_name(name)
     add(paned, index, parent)
-    if position:
-        paned.set_position(int(position))
+    paned.set_position(properties.get(name, int(position)))
+    paned.connect("notify::position", _position_changed, properties)
     paned.show()
     return paned
 
 
 @factory("box")
-def box(parent, index, orientation):
+def box(parent, index, properties, orientation):
     box = Gtk.Box.new(
         Gtk.Orientation.HORIZONTAL
         if orientation == "horizontal"
